############################################################################################################
## Example to interface MODFLOW-2005 and ZONE BUDGET with hydroPSO. This script allows hydroPSO take       #
## control over the execution of MODFLOW-2005 and ZONE BUDGET through the definition of a batch file       #
## (run_me.bat) and a series of simple I/O R scripts.                                                      #
##                                                                                                         #
## Part of the hydroPSO R package                                                                          #
## http://www.rforge.net/hydroPSO/ - http://cran.r-project.org/web/packages/hydroPSO                       #
## Copyright 2011-2012 Mauricio Zambrano-Bigiarini & Rodrigo Rojas                                         #
## Distributed under GPL 2 or later                                                                        #
##                                                                                                         #
## Created by Mauricio Zambrano-Bigiarini and Rodrigo Rojas. 26-Oct-2011.                                  #
## Last saved: 13-Feb-2012                                                                                 #
############################################################################################################

##Loading required libraries.
library(hydroPSO)
library(hydroTSM)
library(hydroGOF)

##Definition of working directory: input, output and model files paths.
model.drty <- "C:/MF2005"
setwd(model.drty )                        

##Customized I/O functions (R scripts) to interface MF2005 with hydroPSO.
source("read.hobs.R")                                                        ### R script to read groundwater head observations from a *.HOB file.
source("read.hsim.R")                                                        ### R script to read simulated equivalents and residuals from a *.LST file and calculate a Gaussian likelihood as customized goodness-of-fit measure.
source("read.wbal.R")                                                        ### R script to read groundwater balance components from a *.BAL file generated by ZONEBUDGET and write them to an output file.
source("read.lik.R")                                                         ### R script to be used as Goodness-of-fit function to interface hydroPSO and MODFLOW-2005.

##Goodness-of-fit function, either customized or pre-defined GOFs of hydroPSO (or hydroGOF).
gof.FUN      <- "read.lik"                                                   ### R function or pre-defined GOF used in hydroPSO (FILL WITH THE NAMES OF THE PREDEFINED GOFs FUCNTIONS).
gof.FUN.args <- list()                                                       ### Additional arguments of gof.FUN (only if required).

##Getting the OBSERVATIONS for the complete period to be calibrated (not strictly necessary for this example).
obs.fname <- "M2.HOB"                                                        ### File containing the observations used for calibration. This could be a time series or a single time-step at different spatial points.
obs.fname <- paste(file.path(model.drty), "/", obs.fname, sep="")            ### Directory with the file containing the observations.
obs       <- read.hobs(fname=obs.fname)                                      ### Vector of observed values.

###MAIN model function
model.FUN="hydromod"                                                         ### Keyword to calibrate a model different from a test function.     
model.FUN.args=list(                                                         ### List with the arguments to be passed to 'model.FUN'.
 model.drty=model.drty,                                                      ### Model directory.  
 param.files=paste(model.drty, "/PSO.in/ParamFiles.txt", sep=""),            ### Name of the file storing the name of the files that have to be modified for each parameter.
 exe.fname="run_me.bat",                                                     ### Name of the executable (or batch) file for the model to be calibrated.
 ###Function for reading 'simulated equivalents' of the model to be calibrated
 out.FUN="read.hsim",                                                        ### Name of the R function to read the output of the model contained in 'fname'.
 out.FUN.args=list(                                                          ### List with the arguments to be passed to 'out.FUN'.
   fname="M2.LST",                                                           ### MODFLOW-2000 list file summarizing the results (local variable for read.hsim.R).
   nobs=42),                                                                 ### Number of observation wells (local variable for read.hsim.R).
 ###Function for comparing simulated equivalents against observations 
 gof.FUN=gof.FUN,                                                            ### Name of the GoF function used to compare simulations and observations.
 gof.FUN.args=gof.FUN.args,                                                  ### List with additional arguments to be passed to 'out.FUN' (additional to 'sim' and 'obs').
 obs=obs                                                                     ### Vector of observed values.
) ##END model.FUN.args

##MAIN PSO ALGORITHM. For hydroPSO fine-tunning parameters, see Zambrano-Bigiarini and Rojas, 2012. 
hydroPSO(
    fn="hydromod",                                                           ### Default name to calibrate a model.
    model.FUN="hydromod",                                                    ### Default keyword of hydroPSO.
    model.FUN.args=model.FUN.args,                                           ### Arguments for the model to be calibrated.
    method="pso",                                                            ### A valid PSO variant: pso, fips, wfips, ipso.
    control=list(
      MinMax="max",                                                          ### Maximization of the objective function (Gaussian Likelihood).
      npart=70,                                                              ### Number of particles.
      maxit=3000,                                                            ### Number of iterations.
      reltol=1e-10,                                                          ### Numeric tolerance to stop successive hydroPSO iterations. 
      c1=0.5+log(2),                                                         ### Acceleration coefficient c1.
      c2=0.5+log(2),                                                         ### Acceleration coefficient c2.
      use.IW=TRUE,IW.type="linear",IW.w=1/(2*log(2)),IW.exp=1,               ### Parameters for defining the inertia weight scheme employed.
      use.TVc1=TRUE,TVc1.type="non-linear",TVc1.rng=c(1.28,1.05),TVc1.exp=1.5,          ### Parameters for defining a time-varying acceleration coefficient c1.
      use.TVlambda=TRUE,TVlambda.type="linear",TVlambda.rng=c(1.0,0.5),TVlambda.exp=1,  ### Parameters for defining a time-varying velocity constraint lambda.
	  drty.out="PSO.70p3000i.rand_TVc1_TVlambda.out",                    ### Directory that will store the results of hydroPSO.
      REPORT=50                                                              ### Interval to report messages to screen.
    ) ##END control options
) ##END MAIN hydroPSO ALGORITHM
